# Elastic stack (ELK) and Grafana on Docker

Run the latest version of the [Elastic stack][elk-stack] and Grafana with Docker and Docker Compose.

It gives you the ability to analyze any data set by using the searching/aggregation capabilities of Elasticsearch and the visualization power of Kibana or Grafana.

Based on the [official Docker images][elastic-docker] from Elastic:

-   ELK
    -   [Elasticsearch](https://github.com/elastic/elasticsearch/tree/main/distribution/docker)
    -   [Logstash](https://github.com/elastic/logstash/tree/main/docker)
    -   [Kibana](https://github.com/elastic/kibana/tree/main/src/dev/build/tasks/os_packages/docker_generator)
-   [Grafana](https://hub.docker.com/r/grafana/grafana)

---

## tl;dr

```sh
docker-compose up setup
```

```sh
docker-compose up
```

![Animated demo](https://user-images.githubusercontent.com/3299086/155972072-0c89d6db-707a-47a1-818b-5f976565f95a.gif)

---

## Philosophy

This readme and config file aim at providing a very simple entry point to use ELK stack and Grafana.
It is based on the [official repo of ELK](https://github.com/deviantony/docker-elk).

---

## Contents

1. [Requirements](#requirements)
    - [Host setup](#host-setup)
    - [Docker Desktop](#docker-desktop)
        - [Windows](#windows)
        - [macOS](#macos)
1. [Usage](#usage)
    - [Bringing up the stack](#bringing-up-the-stack)
    - [Initial setup](#initial-setup)
        - [Setting up user authentication](#setting-up-user-authentication)
        - [Injecting data](#injecting-data)
    - [Cleanup](#cleanup)
1. [Configuration](#configuration)

    - [How to configure Elasticsearch](#how-to-configure-elasticsearch)
    - [How to configure Kibana](#how-to-configure-kibana)
    - [How to configure Logstash](#how-to-configure-logstash)
    - [How to disable paid features](#how-to-disable-paid-features)
    - [How to re-execute the setup](#how-to-re-execute-the-setup)

1. [Extensibility](#extensibility)
    - [How to add plugins](#how-to-add-plugins)
    - [How to enable the provided extensions](#how-to-enable-the-provided-extensions)

## Requirements

### Host setup

-   [Docker Engine][docker-install] version **18.06.0** or newer
-   [Docker Compose][compose-install] version **1.28.0** or newer (including [Compose V2][compose-v2])
-   1.5 GB of RAM

> [!NOTE]
> Especially on Linux, make sure your user has the [required permissions][linux-postinstall] to interact with the Docker daemon.

By default, the stack exposes the following ports:

-   5044: Logstash Beats input
-   50000: Logstash TCP input
-   9600: Logstash monitoring API
-   9200: Elasticsearch HTTP
-   9300: Elasticsearch TCP transport
-   5601: Kibana
-   3000: Grafana

> [!WARNING]
> Elasticsearch's [bootstrap checks][bootstrap-checks] were purposely disabled to facilitate the setup of the Elastic stack in development environments. For production setups, we recommend users to set up their host according to the instructions from the Elasticsearch documentation: [Important System Configuration][es-sys-config].

### Docker Desktop

#### Windows

If you are using the legacy Hyper-V mode of _Docker Desktop for Windows_, ensure [File Sharing][win-filesharing] is
enabled for the `C:` drive.

#### macOS

The default configuration of _Docker Desktop for Mac_ allows mounting files from `/Users/`, `/Volume/`, `/private/`,
`/tmp` and `/var/folders` exclusively. Make sure the repository is cloned in one of those locations or follow the
instructions from the [documentation][mac-filesharing] to add more locations.

### Bringing up the stack

Clone this repository onto the Docker host that will run the stack.

Then, initialize the Elasticsearch users and groups required by docker-elk by executing the command:

```sh
docker-compose up setup
```

If everything went well and the setup completed without error, start the other stack components:

```sh
docker-compose up
```

> [!NOTE]
> You can also run all services in the background (detached mode) by appending the `-d` flag to the above command.

Give Kibana about a minute to initialize, then access the Kibana web UI by opening <http://localhost:5601> in a web browser and use the following (default) credentials to log in:

-   user: _elastic_
-   password: _changeme_

> [!NOTE]
> Upon the initial startup, the `elastic`, `logstash_internal` and `kibana_system` Elasticsearch users are intialized with the values of the passwords defined in the [`.env`](.env) file (_"changeme"_ by default).
>
> The first one is the [built-in superuser][builtin-users], the other two are used by Kibana and Logstash respectively to communicate with Elasticsearch.
>
> This task is only performed during the _initial_ startup of the stack. To change users' passwords _after_ they have been initialized, please refer to the instructions in the next section.

### Initial setup

#### Injecting data

Launch the Kibana web UI by opening <http://localhost:5601> in a web browser, and use the following credentials to log
in:

-   user: _elastic_
-   password: _\<your generated elastic password>_

Now that the stack is fully configured, you can go ahead and inject some log entries.
You can also load the sample data provided by your Kibana installation.

### Cleanup

Elasticsearch data is persisted inside a volume by default.

In order to entirely shutdown the stack and remove all persisted data, use the following Docker Compose command:

```sh
docker-compose down -v
```

## Configuration

> [!IMPORTANT]
> Configuration is not dynamically reloaded, you will need to restart individual components after any configuration change.

### How to configure Elasticsearch

The Elasticsearch configuration is stored in [`elasticsearch/config/elasticsearch.yml`][config-es].

You can also specify the options you want to override by setting environment variables inside the Compose file:

```yml
elasticsearch:
    environment:
        network.host: _non_loopback_
        cluster.name: my-cluster
```

Please refer to the following documentation page for more details about how to configure Elasticsearch inside Docker containers: [Install Elasticsearch with Docker][es-docker].

### How to configure Kibana

The Kibana default configuration is stored in [`kibana/config/kibana.yml`][config-kbn].

You can also specify the options you want to override by setting environment variables inside the Compose file:

```yml
kibana:
    environment:
        SERVER_NAME: kibana.example.org
```

Please refer to the following documentation page for more details about how to configure Kibana inside Docker containers: [Install Kibana with Docker][kbn-docker].

### How to configure Logstash

The Logstash configuration is stored in [`logstash/config/logstash.yml`][config-ls].

You can also specify the options you want to override by setting environment variables inside the Compose file:

```yml
logstash:
    environment:
        LOG_LEVEL: debug
```

Please refer to the following documentation page for more details about how to configure Logstash inside Docker containers: [Configuring Logstash for Docker][ls-docker].

### How to disable paid features

> [!IMPORTANT]
> ELK [Platinum][subscriptions] features are enabled by default for a [trial][license-mngmt] duration of **30 days**. After > this evaluation period, you will retain access to all the free features included in the Open Basic license seamlessly, without manual intervention required, and without losing any data.
>
> Refer to the [How to disable paid > features](#how-to-disable-paid-features) section to opt out of this behaviour.

You can cancel an ongoing trial before its expiry date — and thus revert to a basic license — either from the [License Management][license-mngmt] panel of Kibana, or using Elasticsearch's `start_basic` [Licensing API][license-apis]. Please note that the second option is the only way to recover access to Kibana if the license isn't either switched to `basic` or upgraded before the trial's expiry date.

Changing the license type by switching the value of Elasticsearch's `xpack.license.self_generated.type` setting from `trial` to `basic` (see [License settings][license-settings]) will only work **if done prior to the initial setup.**
After a trial has been started, the loss of features from `trial` to `basic` _must_ be acknowledged using one of the two methods described in the first paragraph.

### How to re-execute the setup

To run the setup container again and re-initialize all users for which a password was defined inside the `.env` file, simply "up" the `setup` Compose service again:

```console
$ docker-compose up setup
 ⠿ Container docker-elk-elasticsearch-1  Running
 ⠿ Container docker-elk-setup-1          Created
Attaching to docker-elk-setup-1
...
docker-elk-setup-1  | [+] User 'monitoring_internal'
docker-elk-setup-1  |    ⠿ User does not exist, creating
docker-elk-setup-1  | [+] User 'beats_system'
docker-elk-setup-1  |    ⠿ User exists, setting password
docker-elk-setup-1 exited with code 0
```

## Extensibility

### How to add plugins

To add plugins to any ELK component you have to:

1. Add a `RUN` statement to the corresponding `Dockerfile` (eg. `RUN logstash-plugin install logstash-filter-json`)
1. Add the associated plugin code configuration to the service configuration (eg. Logstash input/output)
1. Rebuild the images using the `docker-compose build` command

### How to enable the provided extensions

A few extensions are available inside the [`extensions`](extensions) directory. These extensions provide features which are not part of the standard Elastic stack, but can be used to enrich it with extra integrations.

The documentation for these extensions is provided inside each individual subdirectory, on a per-extension basis. Some of them require manual changes to the default ELK configuration.

[elk-stack]: https://www.elastic.co/what-is/elk-stack
[elastic-docker]: https://www.docker.elastic.co/
[subscriptions]: https://www.elastic.co/subscriptions
[es-security]: https://www.elastic.co/guide/en/elasticsearch/reference/current/security-settings.html
[license-settings]: https://www.elastic.co/guide/en/elasticsearch/reference/current/license-settings.html
[license-mngmt]: https://www.elastic.co/guide/en/kibana/current/managing-licenses.html
[license-apis]: https://www.elastic.co/guide/en/elasticsearch/reference/current/licensing-apis.html
[elastdocker]: https://github.com/sherifabdlnaby/elastdocker
[docker-install]: https://docs.docker.com/get-docker/
[compose-install]: https://docs.docker.com/compose/install/
[compose-v2]: https://docs.docker.com/compose/compose-v2/
[linux-postinstall]: https://docs.docker.com/engine/install/linux-postinstall/
[bootstrap-checks]: https://www.elastic.co/guide/en/elasticsearch/reference/current/bootstrap-checks.html
[es-sys-config]: https://www.elastic.co/guide/en/elasticsearch/reference/current/system-config.html
[es-heap]: https://www.elastic.co/guide/en/elasticsearch/reference/current/important-settings.html#heap-size-settings
[win-filesharing]: https://docs.docker.com/desktop/settings/windows/#file-sharing
[mac-filesharing]: https://docs.docker.com/desktop/settings/mac/#file-sharing
[builtin-users]: https://www.elastic.co/guide/en/elasticsearch/reference/current/built-in-users.html
[ls-monitoring]: https://www.elastic.co/guide/en/logstash/current/monitoring-with-metricbeat.html
[sec-cluster]: https://www.elastic.co/guide/en/elasticsearch/reference/current/secure-cluster.html
[connect-kibana]: https://www.elastic.co/guide/en/kibana/current/connect-to-elasticsearch.html
[index-pattern]: https://www.elastic.co/guide/en/kibana/current/index-patterns.html
[config-es]: ./elasticsearch/config/elasticsearch.yml
[config-kbn]: ./kibana/config/kibana.yml
[config-ls]: ./logstash/config/logstash.yml
[es-docker]: https://www.elastic.co/guide/en/elasticsearch/reference/current/docker.html
[kbn-docker]: https://www.elastic.co/guide/en/kibana/current/docker.html
[ls-docker]: https://www.elastic.co/guide/en/logstash/current/docker-config.html
[upgrade]: https://www.elastic.co/guide/en/elasticsearch/reference/current/setup-upgrade.html
